#include <cstdio>
#include <list>
#include <vector>
using namespace std;

struct hrana
{
  int id; // číslo hrany
  int delta; // rozdíl počtu bonbónů
  int odkud, kam; // spojené vrcholy
};

struct vrchol
{
  bool zpracovany;     // zda je vrchol v dříve zpracované komponentě
  bool akt_komponenta; // zda je vrchol v aktuálně zpracovávané komponentě
  bool navstiveny;     // zda jsme vrchol navštívili v aktuálním průchodu
  hrana *P_posl;       // poslední hrana na cestě P_v do tohoto vrcholu v
  int s_v;             // součet ohodnocení hran na cestě P_v
  hrana *Z_prv;        // první hrana na cestě Z_v z tohoto vrcholu v
  int sp_v;            // Součet ohodnocení hran na cestě Z_v
  list<hrana *> vstupuji, vychazi;  // příchozí a odchozí hrany
  vrchol(void) : zpracovany(false), akt_komponenta(false), navstiveny(false),
		 P_posl(0), s_v(0), Z_prv(0), sp_v(0), vstupuji(), vychazi()
    { }
};

// Seznam hran a vrcholů grafu
static vector<hrana> hrany;
static vector<vrchol> vrcholy;

// Smazání značek navštívenosti ve zpracovávané komponentě k
static void smaz_znacky(const list<int> &k)
{
  for (int v : k)
    vrcholy[v].navstiveny = false;
}

// Nalezení cest P_v a Z_v.
static void najdi_P_v(int v, int delta = 0, hrana *posl = 0)
{
  if (!vrcholy[v].akt_komponenta || vrcholy[v].navstiveny)
    return;
  vrcholy[v].navstiveny = true;
  vrcholy[v].s_v = delta;
  vrcholy[v].P_posl = posl;

  for (hrana *e : vrcholy[v].vychazi)
    if (e != posl)
      najdi_P_v(e->kam, delta + e->delta, e);
}

static void najdi_Z_v(int v, int delta = 0, hrana *posl = 0)
{
  if (!vrcholy[v].akt_komponenta || vrcholy[v].navstiveny)
    return;
  vrcholy[v].navstiveny = true;
  vrcholy[v].sp_v = delta;
  vrcholy[v].Z_prv = posl;

  for (hrana *e : vrcholy[v].vstupuji)
    if (e != posl)
      najdi_Z_v(e->odkud, delta + e->delta, e);
}

// Vypíše cestu P_v nebo Z_v
static void vypis_P_v(int v)
{
  hrana *e = vrcholy[v].P_posl;

  if (e)
    {
      vypis_P_v(e->odkud);
      printf("%d ", e->id);
    }
}

static void vypis_Z_v(int v)
{
  hrana *e = vrcholy[v].Z_prv;

  if (e)
    {
      printf("%d ", e->id);
      vypis_Z_v(e->kam);
    }
}

/* Ověří, zda v komponentě k existuje uzavřený tah s nenulovým součtem ohodnocení.
   Pokud ano, vypíše ho a vrátí true, jinak vrátí false. */
static bool nenulovy_tah(const list<int> &k)
{
  for (int v : k)
    vrcholy[v].akt_komponenta = true;

  int v0 = k.front();
  najdi_P_v(v0);
  smaz_znacky(k);
  najdi_Z_v(v0);
  smaz_znacky(k);

  for (int v : k)
    if (vrcholy[v].s_v + vrcholy[v].sp_v != 0)
      {
	vypis_P_v(v);
	vypis_Z_v(v);
	printf("\n");
	return true;
      }

  for (int v : k)
    for (hrana *e : vrcholy[v].vychazi)
      {
	int u = e->kam;
	if (vrcholy[u].akt_komponenta
	    && vrcholy[v].s_v + e->delta + vrcholy[u].sp_v != 0)
	  {
	    vypis_P_v(v);
	    printf("%d ", e->id);
	    vypis_Z_v(u);
	    printf("\n");
	    return true;
	  }
      }

  for (int v : k)
    vrcholy[v].akt_komponenta = false;

  return false;
}

/* Provede prohledání do hloubky a vrátí vrcholy v opačném pořadí dle
   konců jejich průchodů. */
static void poradi_dle_prohledani(int v, list<int> &poradi)
{
  if (vrcholy[v].navstiveny)
    return;
  vrcholy[v].navstiveny = true;

  for (hrana *e : vrcholy[v].vychazi)
    poradi_dle_prohledani(e->kam, poradi);

  poradi.push_front(v);
}

static void poradi_dle_prohledani(list<int> &poradi)
{
  int n = vrcholy.size();
  for (int v = 0; v < n; v++)
    poradi_dle_prohledani(v, poradi);
  for (int v = 0; v < n; v++)
    vrcholy[v].navstiveny = false;
}

/* Najde komponentu silné souvislosti obsahující v. */
static void najdi_komponentu(int v, list<int> &k)
{
  if (vrcholy[v].zpracovany)
    return;

  k.push_back(v);
  vrcholy[v].zpracovany = true;

  for (hrana *e : vrcholy[v].vstupuji)
    najdi_komponentu(e->odkud, k);

}

/* Pro každou komponentu silné souvislosti spustí hledání tahu
   s nenulovým součtem ohodnocení. */
static bool nenulovy_tah(void)
{
  list<int> poradi;
  poradi_dle_prohledani(poradi);

  for (int v : poradi)
    if (!vrcholy[v].zpracovany)
      {
	list<int> k;
	najdi_komponentu(v, k);
	if (nenulovy_tah(k))
	  return true;
      }

  return false;
}

int main(void)
{
  int n, m;
  scanf("%d%d", &n, &m);
  vrcholy.resize(n);
  hrany.resize(m);

  for (int id = 1; id <= m; ++id)
    {
      int f, t, a, b;
      scanf("%d%d%d%d", &f, &t, &a, &b);
      f--; t--;

      hrana *e = &hrany[id - 1];
      e->id = id;
      e->odkud = f;
      e->kam = t;
      e->delta = a - b;

      vrcholy[f].vychazi.push_back(e);
      vrcholy[t].vstupuji.push_back(e);
    }

  if (!nenulovy_tah())
    printf("0\n");
  return 0;
}
